package cz.uhk.bp.locationrecorder.Activities;

import static cz.uhk.bp.locationrecorder.Services.LocationService.provideLocation;
import static cz.uhk.bp.locationrecorder.Services.Services.findCoordinatesByMacAddress;
import static cz.uhk.bp.locationrecorder.Services.Services.formatToGPX;
import static cz.uhk.bp.locationrecorder.Services.Services.getCompleteDateFormat;
import static cz.uhk.bp.locationrecorder.Services.Services.getCurrentTime;
import static cz.uhk.bp.locationrecorder.Services.Services.getGPXDateFormat;
import static cz.uhk.bp.locationrecorder.Services.Services.loadFromJson;
import static cz.uhk.bp.locationrecorder.Services.Services.readFile;
import static cz.uhk.bp.locationrecorder.Services.Services.sortWifiPoints;
import static cz.uhk.bp.locationrecorder.Services.Services.writeFile;
import static cz.uhk.bp.locationrecorder.Services.WifiScanner.scanWifi;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

import android.content.Context;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import cz.uhk.bp.locationrecorder.GeoCoordinates;
import cz.uhk.bp.locationrecorder.R;
import cz.uhk.bp.locationrecorder.Services.LocationService;
import cz.uhk.bp.locationrecorder.WifiPoint;

public class LocationRecorder extends AppCompatActivity {

    private TextView timeTextView;
    private TextView infoTextView;
    private TextView wifiPointsTextView;
    private TextView locationTextView;
    private Button startButton;

    private WifiManager wifiManager;
    private LocationManager locationManager;

    private static final int LOCATION_PERMISSION_REQUEST_CODE = 1000;

    private Handler handler;
    private Handler recordingHandler;
    private Handler wifiScanHandler;

    private GeoCoordinates actualGeoCoordinates;
    //    private GeoCoordinates actualWifiCoordinates;
    private List<WifiPoint> wifiPoints;
    private List<WifiPoint> actualWifiPoints;

    private boolean actionButtonClicked = false;

    private List<GeoCoordinates> recordedGeoCoordinates;
    private List<GeoCoordinates> recordedWifiCoordinates;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_location_recorder);

        //screen keeps on
        View rootLayout = findViewById(R.id.textView_wifiPoints);
        rootLayout.setKeepScreenOn(true);

        timeTextView = findViewById(R.id.textView_time);
        infoTextView = findViewById(R.id.textView_info);
        wifiPointsTextView = findViewById(R.id.textView_wifiPoints);
        locationTextView = findViewById(R.id.textView_location);
        startButton = findViewById(R.id.button_action);

        // managers
        wifiManager = (WifiManager) getApplicationContext().getSystemService(Context.WIFI_SERVICE);
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);

        //handlers
        handler = new Handler();
        recordingHandler = new Handler();
        wifiScanHandler = new Handler();

        actualGeoCoordinates = new GeoCoordinates();
        //lists
        wifiPoints = new ArrayList<>();
        recordedGeoCoordinates = new ArrayList<>();
        recordedWifiCoordinates = new ArrayList<>();

        //listeners
        startButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!actionButtonClicked) {
                    //start actions
                    if (actualGeoCoordinates.getLatitude() != 0) {
                        String jsonWifiPoints = readFile(getApplicationContext(), "wifiPoints.json");
                        // gnss + wifi
                        if (jsonWifiPoints != null) {
                            wifiPoints = loadFromJson(jsonWifiPoints);
                            infoTextView.setText("Started recording position GNSS + Wi-Fi");
                            startWifiRecordingGeoCoordinates();
                            startRecordingGeoCoordinates();

                        } else {
                            //only gnss
                            Toast.makeText(getApplicationContext(), "File with Wi-Fi points not found,\n cannot start Wi-Fi location record", Toast.LENGTH_SHORT).show();
                            infoTextView.setText("Started recording position only GNSS");
                            startRecordingGeoCoordinates();
                        }
                        actionButtonClicked = true;
                        startButton.setText("Stop");

                    } else {
                        Toast.makeText(getApplicationContext(), "Actual geo coordinates was not updated yet\n Cannot start collecting Wi-Fi points", Toast.LENGTH_SHORT).show();
                    }
                } else {
                    //end actions
                    infoTextView.setText("stop clicked |" + "Wifi: " + recordedWifiCoordinates.size() + " | Geo : " + recordedGeoCoordinates.size());
                    if (wifiPoints != null) {
                        boolean gpsFile = false;
                        boolean wifiFile = false;

                        stopRecordingCoordinates();

                        String completeDate = getCompleteDateFormat();
                        // file geo gpx
                        if (!recordedGeoCoordinates.isEmpty()) {
                            String gpxData = formatToGPX(recordedGeoCoordinates);
                            String fileName = "GPS_" + completeDate + ".gpx";
                            File file = writeFile(getApplicationContext(), gpxData, fileName);
                            if (file != null) {
                                gpsFile = true;
                            }
                            recordedGeoCoordinates.clear();
                        }
                        // file wifi gpx
                        if (!recordedWifiCoordinates.isEmpty()) {
                            String gpxWifiData = formatToGPX(recordedWifiCoordinates);
                            String fileName = "WiFi_" + completeDate + ".gpx";
                            File file = writeFile(getApplicationContext(), gpxWifiData, fileName);
                            if (file != null) {
                                wifiFile = true;
                            }
                            recordedWifiCoordinates.clear();
                        }

                        if (gpsFile && wifiFile) {
                            infoTextView.setText("Files were saved");
                        }
                        if (!gpsFile && !wifiFile) {
                            infoTextView.setText("No file was saved");
                        }

                        if (gpsFile && !wifiFile) {
                            infoTextView.setText("GNSS file was saved only");
                        }

                        if (!gpsFile && wifiFile) {
                            infoTextView.setText("Wi-Fi file was saved");

                        }
                        wifiPoints.clear();
                    }
                    actionButtonClicked = false;
                    startButton.setText("Start");
                }


            }
        });

//
        // location permission and start updating the UI
        requestLocationPermissionAndStartUpdating();
    }


    private void requestLocationPermissionAndStartUpdating() {
        if (ActivityCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {

            // permission not granted
            wifiPointsTextView.setText("fine location accessed");
            ActivityCompat.requestPermissions(this, new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION}, LOCATION_PERMISSION_REQUEST_CODE);

        } else {

            // permission granted, start updating the UI
            wifiPointsTextView.setText("fine granted");
            updateTime();
            startLocationUpdates();
            startWifiScan();
        }
    }

    private void startLocationUpdates() {
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                provideLocation(locationManager, new LocationService.LocationCallback() {
                    @Override
                    public void onLocationChanged(GeoCoordinates geoCoordinates) {
                        actualGeoCoordinates = geoCoordinates;

                        String time = getCurrentTime();
                        String locationText = String.format("Time: %s\nLatitude: %s\nLongitude: %s\nAccuracy: %s\n", time, actualGeoCoordinates.getLatitude(), actualGeoCoordinates.getLongitude(), actualGeoCoordinates.getAccuracy());
                        locationTextView.setText(locationText);

                    }
                });

                // schedule task run again after 1 second
                handler.postDelayed(this, 1000);
            }
        }, 1000);


    }


    private void startWifiScan() {
        wifiScanHandler.postDelayed(new Runnable() {
            @Override
            public void run() {
                actualWifiPoints = scanWifi(wifiManager);
                if (actualWifiPoints != null) {

                    actualWifiPoints = sortWifiPoints(actualWifiPoints);

                    StringBuilder sb = new StringBuilder();
                    // add current time
                    sb.append(getCurrentTime()).append("\n");

                    for (WifiPoint wifiPoint : actualWifiPoints) {
                        sb.append(wifiPoint.getName()).append(" \n    ").append(wifiPoint.getMacAddress())
                                .append("| ").append(wifiPoint.getSignalStrength()).append("dBm\n");
                    }

                    wifiPointsTextView.setText(sb.toString());
                    // scan every second
                    wifiScanHandler.postDelayed(this, 1000);
                } else {
                    wifiPointsTextView.setText("Cannot get Wi-Fi points");
                }
            }
        }, 1000);


    }


    private void startRecordingGeoCoordinates() {
        recordingHandler.postDelayed(new Runnable() {
            @Override
            public void run() {

                provideLocation(locationManager, new LocationService.LocationCallback() {
                    @Override
                    public void onLocationChanged(GeoCoordinates geoCoordinates) {
                        actualGeoCoordinates = geoCoordinates;

                    }
                });

                actualGeoCoordinates.setTime(getGPXDateFormat());
                recordedGeoCoordinates.add(actualGeoCoordinates);

                infoTextView.setText("Geo: " + recordedGeoCoordinates.size() + " | Wifi: " + recordedWifiCoordinates.size());

                // schedule task run again after 1 second
                recordingHandler.postDelayed(this, 1000);
            }
            // initial delay of 1 second
        }, 1000);
    }


    private void startWifiRecordingGeoCoordinates() {
        recordingHandler.postDelayed(new Runnable() {
            @Override
            public void run() {
                GeoCoordinates actualWifiCoordinates = findCoordinatesByMacAddress(actualWifiPoints, wifiPoints);
                if (actualWifiCoordinates != null) {

                    // returns same time for every coordinate
//                        actualWifiCoordinates.setTime(getGPXDateFormat());
//                        recordedWifiCoordinates.add(actualWifiCoordinates);
                    // returns current time for every coordinate
                    GeoCoordinates actualWifiCoordinates2 = new GeoCoordinates();
                    actualWifiCoordinates2.setLatitude(actualWifiCoordinates.getLatitude());
                    actualWifiCoordinates2.setLongitude(actualWifiCoordinates.getLongitude());
                    actualWifiCoordinates2.setAccuracy(actualWifiCoordinates.getAccuracy());
                    actualWifiCoordinates2.setTime(getGPXDateFormat());
                    recordedWifiCoordinates.add(actualWifiCoordinates2);


                }
                recordingHandler.postDelayed(this, 1000);
            }
            // initial delay of 1 second
        }, 1000);
    }


    private void stopRecordingCoordinates() {
        // stop recording
        recordingHandler.removeCallbacksAndMessages(null);
    }


    private void updateTime() {
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                timeTextView.setText(getCurrentTime() + " \n");
                // scan every second
                handler.postDelayed(this, 1000);
            }
        }, 1000);
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == LOCATION_PERMISSION_REQUEST_CODE) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                // permission granted, start updating the UI
                updateTime();
                startLocationUpdates();
                startWifiScan();
            } else {
                // permission denied
                infoTextView.setText("Permission denied, app cannot work");
                Toast.makeText(getApplicationContext(), "Permission denied, app cannot work", Toast.LENGTH_SHORT).show();
            }
        }


    }

}