package cz.uhk.bp.locationrecorder.Activities;

import static cz.uhk.bp.locationrecorder.Services.LocationService.provideLocation;
import static cz.uhk.bp.locationrecorder.Services.Services.getCurrentTime;
import static cz.uhk.bp.locationrecorder.Services.Services.insertUpdateWifiPoint;
import static cz.uhk.bp.locationrecorder.Services.Services.loadFromJson;
import static cz.uhk.bp.locationrecorder.Services.Services.readFile;
import static cz.uhk.bp.locationrecorder.Services.Services.saveToJson;
import static cz.uhk.bp.locationrecorder.Services.Services.writeFile;
import static cz.uhk.bp.locationrecorder.Services.WifiScanner.scanWifi;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

import android.content.Context;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import cz.uhk.bp.locationrecorder.GeoCoordinates;
import cz.uhk.bp.locationrecorder.R;
import cz.uhk.bp.locationrecorder.Services.LocationService;
import cz.uhk.bp.locationrecorder.WifiPoint;

public class WifiCollector extends AppCompatActivity {


    private TextView timeTextView;
    private TextView infoTextView;
    private TextView wifiPointsTextView;
    private TextView locationTextView;
    private Button actiontButton;

    private WifiManager wifiManager;
    private LocationManager locationManager;

    private static final int LOCATION_PERMISSION_REQUEST_CODE = 1000;
    private Handler handler;
    private Handler recordingHandler;

    private GeoCoordinates actualGeoCoordinates;
    private List<WifiPoint> wifiPoints;
    private List<WifiPoint> actualWifiPoints;

    private boolean saveActualWifiPoints = false;
    private boolean actionButtonClicked = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_wifi_collector);


        //screen keeps on
        View rootLayout = findViewById(R.id.textView_wifiPoints);
        rootLayout.setKeepScreenOn(true);

        timeTextView = findViewById(R.id.textView_time);
        infoTextView = findViewById(R.id.textView_info);
        wifiPointsTextView = findViewById(R.id.textView_wifiPoints);
        locationTextView = findViewById(R.id.textView_location);
        actiontButton = findViewById(R.id.button_action);


        wifiManager = (WifiManager) getApplicationContext().getSystemService(Context.WIFI_SERVICE);
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);


        handler = new Handler();
        recordingHandler = new Handler();


        actualGeoCoordinates = new GeoCoordinates();
        wifiPoints = new ArrayList<>();

        actiontButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!actionButtonClicked) {
                    //start actions
                    if (actualGeoCoordinates.getLatitude() != 0) {
                        String jsonWifiPoints = readFile(getApplicationContext(), "wifiPoints.json");
                        if (jsonWifiPoints != null) {
                            wifiPoints = loadFromJson(jsonWifiPoints);
                        } else {
                            Toast.makeText(getApplicationContext(), "File with Wi-Fi points not found,\nstarted new list", Toast.LENGTH_SHORT).show();
                            wifiPoints = new ArrayList<>();
                        }
                        infoTextView.setText("Collecting started| Actual: " + actualWifiPoints.size() + " | Total: " + wifiPoints.size());
                        startCollectingWifiPoints();

                        actionButtonClicked = true;
                        actiontButton.setText("Stop");
                    } else {
                        infoTextView.setText("Cannot start, actual geo coordinates has not been updated yet ");
                        Toast.makeText(getApplicationContext(), "Actual geo coordinates has not been updated yet\n Cannot start collecting Wi-Fi points", Toast.LENGTH_SHORT).show();
                    }

                } else {
                    //stop actions
                    infoTextView.setText("Stop clicked | " + "Total: " + wifiPoints.size());
                    stopCollectingWifiPoints();

                    wifiPointsTextView.setText(saveToJson(wifiPoints));
                    String jsonString = saveToJson(wifiPoints);

                    File file = writeFile(getApplicationContext(), jsonString, "wifiPoints.json");
                    if (file != null) {
                        infoTextView.setText("Wi-Fi points saved | " + "Total: " + wifiPoints.size() + " saved");
                    } else {
                        infoTextView.setText("Wi-Fi points cannot be saved");
                    }
                    wifiPoints.clear();

                    actionButtonClicked = false;
                    actiontButton.setText("Start");
                }
            }

        });


        //location permission and start updating the UI
        requestLocationPermissionAndStartUpdating();
    }


    private void requestLocationPermissionAndStartUpdating() {
        if (ActivityCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // Permission is not granted, request it
            wifiPointsTextView.setText("fine location accessed");
            ActivityCompat.requestPermissions(this, new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION}, LOCATION_PERMISSION_REQUEST_CODE);

        } else {
            // permission granted, start updating the UI
            updateTime();
            startLocationUpdates();
            startWifiScan();
        }
    }


    private void startLocationUpdates() {
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                provideLocation(locationManager, new LocationService.LocationCallback() {
                    @Override
                    public void onLocationChanged(GeoCoordinates geoCoordinates) {
                        actualGeoCoordinates = geoCoordinates;

                        String time = getCurrentTime();
                        String locationText = String.format("Time: %s\nLatitude: %s\nLongitude: %s\nAccuracy: %s\n", time, actualGeoCoordinates.getLatitude(), actualGeoCoordinates.getLongitude(), actualGeoCoordinates.getAccuracy());
                        locationTextView.setText(locationText);

                    }
                });


                // schedule task to run again after 1 second
                handler.postDelayed(this, 1000);
            }
        }, 1000);

    }


    private void startWifiScan() {
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                actualWifiPoints = scanWifi(wifiManager, actualGeoCoordinates);
                if (actualWifiPoints != null) {

                    StringBuilder sb = new StringBuilder();
                    sb.append(getCurrentTime()).append("\n"); // Add current time

                    for (WifiPoint wifiPoint : actualWifiPoints) {
                        sb.append(wifiPoint.getName()).append(" \n    ").append(wifiPoint.getMacAddress())
                                .append("| ").append(wifiPoint.getSignalStrength()).append("dBm\n");
                    }

                    wifiPointsTextView.setText(sb.toString());


                    // scan every second
                    handler.postDelayed(this, 1000);
                } else {
                    wifiPointsTextView.setText("Cannot get Wi-Fi points");
                }
            }

        }, 1000);


    }


    private void startCollectingWifiPoints() {
        recordingHandler.postDelayed(new Runnable() {
            @Override
            public void run() {

                if (actualWifiPoints != null) {
                    wifiPoints = insertUpdateWifiPoint(wifiPoints, actualWifiPoints);
                    infoTextView.setText("Actual visible: " + actualWifiPoints.size() + " | Total: " + wifiPoints.size());
                    // scan every second
                    recordingHandler.postDelayed(this, 1000);
                } else {
                    wifiPointsTextView.setText("Cannot get Wi-Fi points");
                }
            }

        }, 1000);


    }

    private void stopCollectingWifiPoints() {
        // stop recording
        recordingHandler.removeCallbacksAndMessages(null);
    }


    private void updateTime() {
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                timeTextView.setText(getCurrentTime() + " \n");
                // scan every second
                handler.postDelayed(this, 1000);
            }
        }, 1000);
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == LOCATION_PERMISSION_REQUEST_CODE) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                // permission granted, start updating UI
                updateTime();
                startLocationUpdates();
                startWifiScan();
            } else {
                // permission denied
                infoTextView.setText("Permission denied, app cannot work");
                Toast.makeText(getApplicationContext(), "Permission denied, app cannot work", Toast.LENGTH_SHORT).show();
            }
        }


    }
}