package cz.uhk.bp.locationrecorder.Services;

import android.content.Context;
import android.icu.text.SimpleDateFormat;
import android.icu.util.TimeZone;
import android.os.Environment;
import android.util.Log;
import android.widget.Toast;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import cz.uhk.bp.locationrecorder.GeoCoordinates;
import cz.uhk.bp.locationrecorder.WifiPoint;

public class Services {


    public static List<WifiPoint> insertUpdateWifiPoint(List<WifiPoint> wifiPoints, List<WifiPoint> actualWifiPoints) {
        if (!wifiPoints.isEmpty()) {
            for (WifiPoint actualWifiPoint : actualWifiPoints) {
                boolean found = false;
                for (WifiPoint wifiPoint : wifiPoints) {
                    if (wifiPoint.getMacAddress().equals(actualWifiPoint.getMacAddress())) {
                        found = true;
                        if (actualWifiPoint.getSignalStrength() < wifiPoint.getSignalStrength()) {
                            // update coordinates - signal strength is better
                            wifiPoint.setGeoCoordinates(actualWifiPoint.getGeoCoordinates());
                        }
                        break; // end loop when found
                    }
                }
                if (!found) {
                    // add actualWifiPoint to wifiPoints if it was not found
                    wifiPoints.add(actualWifiPoint);
                }
            }
        } else {
            wifiPoints.addAll(actualWifiPoints);
        }
        return wifiPoints;
    }

    //time operations
    public static String getCurrentTime() {
        // current time in HH:mm:ss format
        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("HH:mm:ss", Locale.getDefault());
        String currentTime = sdf.format(new Date());
        return currentTime;
    }

    public static String getGPXDateFormat() {
        // date format - the GPX standard
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'", Locale.getDefault());
        dateFormat.setTimeZone(TimeZone.getTimeZone("Europe/Prague")); // Set the time zone to Prague
        String date = dateFormat.format(new Date());
        return date; // Return current date formatted as per the GPX standard
    }

    public static String getCompleteDateFormat() {
        //  current time in "year:month:day:hh:mm:ss" format
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy:MM:dd-HH:mm:ss", Locale.getDefault());
        String currentTime = sdf.format(new Date());
        return currentTime;
    }

    // return json string
    public static String saveToJson(List<WifiPoint> wifiPoints) {
        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        String json = gson.toJson(wifiPoints);

        return json;
    }

    // read json string
    public static List<WifiPoint> loadFromJson(String json) {
        Gson gson = new Gson();
        Type listType = new TypeToken<List<WifiPoint>>() {
        }.getType();

        return gson.fromJson(json, listType);
    }


    // external storage app data
    public static File writeFile(Context context, String data, String fileMame) {
        // check external storage available for writing
        if (isExternalStorageWritable()) {
            //get root directory
            File externalStorageDir = context.getExternalFilesDir(null);
            if (externalStorageDir != null) {
                //create file
                File file = new File(externalStorageDir, fileMame);
                try {
                    FileOutputStream stream = new FileOutputStream(file);
                    stream.write(data.getBytes());
                    stream.close();
                    return file;
                } catch (IOException e) {
                    Toast.makeText(context, "File cannot be saved", Toast.LENGTH_SHORT).show();
                    e.printStackTrace();
                    return null;
                }
            } else {
                Log.e("writeFile", "External storage directory is null");
                return null;
            }
        } else {
            Log.e("writeFile", "External storage is not writable");
            return null;
        }
    }


    // check if storage is available for writing operations
    public static boolean isExternalStorageWritable() {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals(state);
    }


    public static String readFile(Context context, String fileName) {
        // check external storage available for reading operations
        if (isExternalStorageReadable()) {
            // get root directory
            File externalStorageDir = context.getExternalFilesDir(null);
            if (externalStorageDir != null) {
                //create file
                File file = new File(externalStorageDir, fileName);
                if (file.exists()) {
                    try {
                        FileInputStream stream = new FileInputStream(file);
                        InputStreamReader inputStreamReader = new InputStreamReader(stream);
                        BufferedReader reader = new BufferedReader(inputStreamReader);
                        StringBuilder stringBuilder = new StringBuilder();
                        String line;
                        while ((line = reader.readLine()) != null) {
                            stringBuilder.append(line);
                        }
                        reader.close();
                        return stringBuilder.toString();
                    } catch (IOException e) {
                        e.printStackTrace();
                        return null;
                    }
                } else {
                    Log.e("readFile", "File does not exist");
                    return null;
                }
            } else {
                Log.e("readFile", "External storage directory is null");
                return null;
            }
        } else {
            Log.e("readFile", "External storage is not readable");
            return null;
        }

    }

    // storage is available for reading operations
    public static boolean isExternalStorageReadable() {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals(state) ||
                Environment.MEDIA_MOUNTED_READ_ONLY.equals(state);
    }

    //sort wifi points
    public static List<WifiPoint> sortWifiPoints(List<WifiPoint> wifiPoints) {
        Collections.sort(wifiPoints, new WifiPointSignalStrengthComparator());
        return wifiPoints;
    }

    //find wifi coordinates
    public static GeoCoordinates findCoordinatesByMacAddress(List<WifiPoint> actualWifiPoints, List<WifiPoint> WifiPoints) {
        for (WifiPoint actualWifiPoint : actualWifiPoints) {
            // Check if the MAC address exists in the all WiFi points list
            for (WifiPoint wifiPoint : WifiPoints) {
                if (wifiPoint.getMacAddress().equals(actualWifiPoint.getMacAddress())) {
                    // Found matching MAC address, return the coordinates
                    return wifiPoint.getGeoCoordinates();
                }
            }
        }
        // If nothing found, return null
        return null;
    }

    //format string to gpx format
    public static String formatToGPX(List<GeoCoordinates> coordinatesList) {
        StringBuilder gpxBuilder = new StringBuilder();
        gpxBuilder.append("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>\n");
        gpxBuilder.append("<gpx version=\"1.1\" creator=\"Location Recorder\">\n");

        for (GeoCoordinates coordinates : coordinatesList) {
            gpxBuilder.append("  <wpt lat=\"").append(coordinates.getLatitude())
                    .append("\" lon=\"").append(coordinates.getLongitude()).append("\">\n");
            gpxBuilder.append("    <time>").append(coordinates.getTime()).append("</time>\n");
            gpxBuilder.append("  </wpt>\n");
        }

        gpxBuilder.append("</gpx>\n");

        return gpxBuilder.toString();
    }


}















